'use strict';

var fs = require('fs');
var path = require('path');
var Util = require('./Util.js');

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var slicedToArray = function () {
  function sliceIterator(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"]) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  return function (arr, i) {
    if (Array.isArray(arr)) {
      return arr;
    } else if (Symbol.iterator in Object(arr)) {
      return sliceIterator(arr, i);
    } else {
      throw new TypeError("Invalid attempt to destructure non-iterable instance");
    }
  };
}();

/**
 * Request body parser module
*/

var _class = function () {

    /**
     *@param {string} tempDir - the absolute path to the temp directory
     *@param {string} encoding - the encoding used for files
    */
    function _class(tempDir, encoding) {
        classCallCheck(this, _class);

        this.tempDir = tempDir;
        this.encoding = encoding ? encoding : 'latin1';
    }

    /**
     * return module identity
    */


    createClass(_class, [{
        key: 'assignValue',


        /**
         * assigns value to the given target with the key name
         *@param {Object} target - the target object
         *@param {string} name - the field name
         *@param {string} value - the field value
        */
        value: function assignValue(target, name, value) {
            //field names that end with [] take multiple values
            if (/\[\]$/.test(name)) {
                name = name.replace(/\[\]$/, '');
                if (typeof target[name] === 'undefined') target[name] = [value];else target[name].push(value);
            } else {
                target[name] = value;
            }
        }

        /**
         * parse json content
         *@param {string} string - the request body string
         *@returns {Object}
        */

    }, {
        key: 'parseJSON',
        value: function parseJSON(string) {
            var body = null;
            try {
                body = JSON.parse(string);
            } catch (ex) {
                body = {};
            }
            return body;
        }

        /**
         * parse url encoded request body
         *@param {string} string - the request body string
         *@returns {Object}
        */

    }, {
        key: 'parseUrlEncoded',
        value: function parseUrlEncoded(string) {
            var body = {};
            if (string) {
                var pairs = string.split('&');
                var _iteratorNormalCompletion = true;
                var _didIteratorError = false;
                var _iteratorError = undefined;

                try {
                    for (var _iterator = pairs[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                        var pair = _step.value;

                        var _pair$split = pair.split('='),
                            _pair$split2 = slicedToArray(_pair$split, 2),
                            name = _pair$split2[0],
                            value = _pair$split2[1];

                        this.assignValue(body, decodeURIComponent(name), decodeURIComponent(value));
                    }
                } catch (err) {
                    _didIteratorError = true;
                    _iteratorError = err;
                } finally {
                    try {
                        if (!_iteratorNormalCompletion && _iterator.return) {
                            _iterator.return();
                        }
                    } finally {
                        if (_didIteratorError) {
                            throw _iteratorError;
                        }
                    }
                }
            }
            return body;
        }

        /**
         * parse the query parameters in the url given
         *@param {string} url - the request url
         *@return {Object}
        */

    }, {
        key: 'parseQueryString',
        value: function parseQueryString(url) {
            if (url.indexOf('?') > -1) return this.parseUrlEncoded(url.split('?')[1]);else return {};
        }

        /**
         * clean up temp files
         *@param {Object} files - the files object
        */

    }, {
        key: 'cleanUpTempFiles',
        value: function cleanUpTempFiles(files) {
            /**
             * performs the unlink operation asynchronously
            */
            var unlink = function unlink(file) {
                fs.unlinkSync(file.path);
            };

            /**iterate through and run the unlink process */
            var _iteratorNormalCompletion2 = true;
            var _didIteratorError2 = false;
            var _iteratorError2 = undefined;

            try {
                for (var _iterator2 = Object.entries(files)[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                    var _ref = _step2.value;

                    var _ref2 = slicedToArray(_ref, 2);

                    var file = _ref2[1];

                    if (Util.isArray(file)) file.forEach(unlink);else unlink(file);
                }
            } catch (err) {
                _didIteratorError2 = true;
                _iteratorError2 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion2 && _iterator2.return) {
                        _iterator2.return();
                    }
                } finally {
                    if (_didIteratorError2) {
                        throw _iteratorError2;
                    }
                }
            }
        }

        /**
         * processes and stores file
         *@param {Object} parsedHeaders - the parsed headers
         *@param {string} parsedHeaders.fileName - the file name as captured from the form data
         *@param {string} parsedHeaders.mimeType - file mime type as captured from the form data
         *@param {string} parsedHeaders.encoding - file content transfer encoding type
         *@param {string} content - the file content
         *@returns {Object}
        */

    }, {
        key: 'processFile',
        value: function processFile(parsedHeaders, content) {

            var tempFileName = Util.getRandomText(8) + '.tmp',
                filePath = path.join(this.tempDir, '/', tempFileName);

            Util.mkDirSync(this.tempDir);

            fs.writeFileSync(filePath, content, parsedHeaders.encoding);

            return {
                name: decodeURIComponent(parsedHeaders.fileName).replace(/\.\./g, ''),
                tempName: tempFileName,
                path: filePath,
                mimeType: parsedHeaders.mimeType,
                size: fs.statSync(filePath).size
            };
        }

        /**
         * parses a multipart part headers.
         *@param {Array} headers - array of headers.
         *@returns {Object}
        */

    }, {
        key: 'parsePartHeaders',
        value: function parsePartHeaders(headers) {
            //assume a default value if there are no headers sent
            if (headers.length === 0) return { isFile: false, mimeType: 'text/plain', fileName: '',
                fieldName: Util.getRandomText(6), encoding: this.encoding };

            var result = { isFile: false, mimeType: '', fileName: '', fieldName: '', encoding: this.encoding };
            var _iteratorNormalCompletion3 = true;
            var _didIteratorError3 = false;
            var _iteratorError3 = undefined;

            try {
                for (var _iterator3 = headers[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
                    var header = _step3.value;

                    var _header$split = header.split(/\s*:\s*/),
                        _header$split2 = slicedToArray(_header$split, 2),
                        headerName = _header$split2[0],
                        headerValue = _header$split2[1];

                    switch (headerName.toLowerCase()) {
                        case 'content-disposition':
                            if (/filename="([^"]+)"/.exec(headerValue)) result.fileName = RegExp.$1;

                            headerValue = headerValue.replace(/filename="([^"]+)"/, '');

                            if (/name="([^"]+)"/.exec(headerValue)) result.fieldName = RegExp.$1;else result.fieldName = Util.getRandomText(6);
                            break;

                        case 'content-type':
                            result.isFile = true;
                            result.mimeType = headerValue.split(/;\s*/)[0];
                            break;

                        case 'content-transfer-encoding':
                            result.encoding = headerValue;
                            break;
                    }
                }
            } catch (err) {
                _didIteratorError3 = true;
                _iteratorError3 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion3 && _iterator3.return) {
                        _iterator3.return();
                    }
                } finally {
                    if (_didIteratorError3) {
                        throw _iteratorError3;
                    }
                }
            }

            return result;
        }

        /**
         * parse multipart form data
         *@param {string} string - the request body string
         *@param {string} [boundary] - the multipart boundary token
         *@returns {Object}
         *@see https://www.w3.org/Protocols/rfc1341/7_2_Multipart.html
        */

    }, {
        key: 'parseMultiPart',
        value: function parseMultiPart(string, boundary) {
            var body = {},
                files = {};

            /* istanbul ignore else */
            if (boundary) boundary = '--' + boundary;else if (/(-{2,}[a-z0-9]+)\r\n/i.exec(string)) boundary = RegExp.$1;else return { body: body, files: files };

            //obtain the body parts, discard multipart preamble and epilogue
            var parts = string.split(boundary).slice(1, -1);

            var _iteratorNormalCompletion4 = true;
            var _didIteratorError4 = false;
            var _iteratorError4 = undefined;

            try {
                for (var _iterator4 = parts[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
                    var part = _step4.value;

                    //remove the first and last CRLF
                    part = part.replace(/^\r\n/, '').replace(/\r\n$/, '');

                    var headers = [],
                        content = '';

                    //if there are no headers, assume default values according to the spec
                    /* istanbul ignore if */
                    if (/^\r\n/.test(part)) {
                        content = part.replace(/^\r\n/, '');
                    } else {
                        var lines = part.split(/\r\n/);

                        //a blank line separates the part headers from the part content
                        var separatorLineIndex = lines.indexOf('');

                        //slice out the headers and the content
                        headers = lines.slice(0, separatorLineIndex);
                        content = lines.slice(separatorLineIndex + 1).join('\r\n');
                    }

                    //parse through the headers
                    var parsedHeaders = this.parsePartHeaders(headers);

                    //resolve and assign value
                    var fieldName = parsedHeaders.fieldName,
                        value = parsedHeaders.isFile ? this.processFile(parsedHeaders, content) : content,
                        target = parsedHeaders.isFile ? files : body;

                    this.assignValue(target, fieldName, value);
                }
            } catch (err) {
                _didIteratorError4 = true;
                _iteratorError4 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion4 && _iterator4.return) {
                        _iterator4.return();
                    }
                } finally {
                    if (_didIteratorError4) {
                        throw _iteratorError4;
                    }
                }
            }

            return { body: body, files: files };
        }

        /**
         *@param {Buffer} buffer - the buffer data
         *@param {string} contentType - the request content type
        */

    }, {
        key: 'parse',
        value: function parse(buffer, contentType) {
            var content = buffer.toString(this.encoding),
                tokens = contentType.split(/;\s*/),
                boundary = '';

            switch (tokens[0].toLowerCase()) {
                case 'text/json':
                case 'application/json':
                    return { files: {}, body: this.parseJSON(content) };

                case 'text/plain':
                case 'application/x-www-form-urlencoded':
                    return { files: {}, body: this.parseUrlEncoded(content) };

                case 'multipart/form-data':
                    if (tokens.length === 2 && /boundary\s*=\s*/.test(tokens[1])) boundary = tokens[1].split('=')[1];
                    return this.parseMultiPart(content, boundary);

                default:
                    return { body: {}, files: {} };
            }
        }
    }, {
        key: Symbol.toStringTag,
        get: function get$$1() {
            return 'BodyParser';
        }
    }]);
    return _class;
}();

module.exports = _class;
