'use strict';

var fs = require('fs');
var path = require('path');
var Util = require('./Util.js');
var http = require('http');
var StaticFileServer = require('./StaticFileServer.js');
var RoutingEngine = require('./RoutingEngine.js');
var Router = require('./Router.js');
var BodyParser = require('./BodyParser.js');
var _config = require('../.rsvrc.json');
require('./RServerResponse.js');
var Logger = require('./Logger.js');

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var toConsumableArray = function (arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  } else {
    return Array.from(arr);
  }
};

var _class = function () {

    /**
     *@param {string} configPath - user specified configuration path
    */
    function _class(configPath) {
        classCallCheck(this, _class);


        this.router = new Router(false);
        this.mountedRouters = [];

        /* istanbul ignore else */
        if (require.main) this.entryPath = this.getEntryPath(require.main.filename);else this.entryPath = this.getEntryPath(__dirname);

        this.config = this.resolveConfig(this.entryPath, configPath || '.rsvrc.json');

        this.staticfileServer = new StaticFileServer(this.entryPath, this.config.staticFileDir, this.config.mimeTypes, this.config.defaultDocuments, this.config.cacheControl);

        this.bodyParser = new BodyParser(path.join(this.entryPath, this.config.tempDir, '/'), this.config.encoding);

        this.server = http.createServer();

        this.initServer();
    }

    /**
     * returns the server identity
     *@returns {string}
    */


    createClass(_class, [{
        key: 'getEntryPath',


        /**
         * returns the project's entry path
         *@param {string} knownPath - the known path for the mean time
        */
        value: function getEntryPath(knownPath) {

            if (knownPath.indexOf('node_modules') > 0) return knownPath.split('node_modules/')[0];

            var currentPath = path.join(knownPath, '../');
            while (currentPath !== '/') {
                if (fs.existsSync(currentPath + '/package.json')) return currentPath;

                currentPath = path.join(currentPath, '../');
            }

            /* istanbul ignore next */
            return '';
        }

        /**
         * resolves the configuration object
         *@param {string} entryPath - the project root path
         *@param {string} configPath - the project config file relative path
        */

    }, {
        key: 'resolveConfig',
        value: function resolveConfig(entryPath, configPath) {
            var config = null,
                absConfigPath = path.join(entryPath, configPath);

            if (fs.existsSync(absConfigPath)) config = Util.mergeObjects(_config, require(absConfigPath));else config = _config;

            return config;
        }

        /**
         * mounts a router to the main app
         *@param {string} baseUrl - the router baseUrl
         *@param {Router} router - the router instance
        */

    }, {
        key: 'mount',
        value: function mount(baseUrl, router) {
            //join all routers url
            var routes = router.routes;
            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError = undefined;

            try {
                for (var _iterator = Object.keys(routes)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                    var api = _step.value;

                    var apiRoutes = routes[api];
                    var _iteratorNormalCompletion2 = true;
                    var _didIteratorError2 = false;
                    var _iteratorError2 = undefined;

                    try {
                        for (var _iterator2 = apiRoutes[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                            var apiRoute = _step2.value;

                            apiRoute[0] = path.join(baseUrl, apiRoute[0]);
                        }
                    } catch (err) {
                        _didIteratorError2 = true;
                        _iteratorError2 = err;
                    } finally {
                        try {
                            if (!_iteratorNormalCompletion2 && _iterator2.return) {
                                _iterator2.return();
                            }
                        } finally {
                            if (_didIteratorError2) {
                                throw _iteratorError2;
                            }
                        }
                    }
                }
            } catch (err) {
                _didIteratorError = true;
                _iteratorError = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion && _iterator.return) {
                        _iterator.return();
                    }
                } finally {
                    if (_didIteratorError) {
                        throw _iteratorError;
                    }
                }
            }

            this.mountedRouters.push(router);
        }

        /**
         * runs all request routes
         *@param {RouterEngine} router - the router engine instance
         *@param {string} api - the route api to call
         *@param {Array} routes - routes array
         *@returns {boolean}
        */

    }, {
        key: 'runRoutes',
        value: function runRoutes(engine, api, routes) {
            api = api.toLowerCase();
            var _iteratorNormalCompletion3 = true;
            var _didIteratorError3 = false;
            var _iteratorError3 = undefined;

            try {
                for (var _iterator3 = routes[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
                    var route = _step3.value;

                    engine[api].apply(engine, toConsumableArray(route));
                    if (engine.resolved) return true;
                }
            } catch (err) {
                _didIteratorError3 = true;
                _iteratorError3 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion3 && _iterator3.return) {
                        _iterator3.return();
                    }
                } finally {
                    if (_didIteratorError3) {
                        throw _iteratorError3;
                    }
                }
            }

            return false;
        }

        /**
         * cordinates how routes are executed, including mounted routes
         *@param {string} url - request url
         *@param {string} method - request method
         *@param {http.IncomingMessage} request - the request object
         *@param {RServerResponse} response - the response object
         *@returns {boolean}
        */

    }, {
        key: 'cordinateRoutes',
        value: function cordinateRoutes(url, method, request, response) {
            method = method.toLowerCase();

            var engine = new RoutingEngine(url, method, request, response),
                router = this.router;

            //run on the main router thread
            engine.use(router.middlewares);

            if (this.runRoutes(engine, 'all', router.routes['all'])) return true;
            if (this.runRoutes(engine, method, router.routes[method])) return true;

            //run on the mounted routers' thread
            var _iteratorNormalCompletion4 = true;
            var _didIteratorError4 = false;
            var _iteratorError4 = undefined;

            try {
                for (var _iterator4 = this.mountedRouters[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
                    var mountedRouter = _step4.value;

                    var middlewares = mountedRouter.inheritMiddlewares ? [].concat(toConsumableArray(router.middlewares), toConsumableArray(mountedRouter.middlewares)) : mountedRouter.middlewares;

                    engine.use(middlewares);

                    if (this.runRoutes(engine, 'all', mountedRouter.routes['all'])) return true;
                    /* istanbul ignore else */
                    if (this.runRoutes(engine, method, mountedRouter.routes[method])) return true;
                }
            } catch (err) {
                _didIteratorError4 = true;
                _iteratorError4 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion4 && _iterator4.return) {
                        _iterator4.return();
                    }
                } finally {
                    if (_didIteratorError4) {
                        throw _iteratorError4;
                    }
                }
            }

            return false;
        }

        /**
         * parse all request data
        */

    }, {
        key: 'parseRequestData',
        value: function parseRequestData(request, url, buffers) {
            //parse query
            request.query = this.bodyParser.parseQueryString(url);

            //parse the request body
            if (buffers.length > 0) {
                var result = this.bodyParser.parse(Buffer.concat(buffers), request.headers['content-type']);

                request.body = result.body;
                request.files = result.files;
            }

            //combine the body and query into a data property
            request.data = Object.assign({}, request.query, request.body);
        }

        /**
         * perform house keeping
         *@param {http.IncomingMessage} request - the request object
        */

    }, {
        key: 'onResponseFinish',
        value: function onResponseFinish(request, response) {
            response.staticfileServer = null;

            Logger.logProfile(request, response);
            this.bodyParser.cleanUpTempFiles(request.files);
        }

        /**
         * handle request data event
         *@param {http.IncomingMessage} request - the request object
         *@param {RServerResponse} response - the response object
         *@param {Object} bufferDetails - the buffer details
         *@param {number} bufferDetails.size - the buffer size
         *@param {Array} bufferDetails.buffers - array containing chunks of buffer data
        */

    }, {
        key: 'onRequestEnd',
        value: function onRequestEnd(request, response, bufferDetails) {

            //profile the response time
            response.startTime = Date.now();

            var url = request.url,
                method = request.method,
                headers = request.headers;


            request.files = {};
            request.query = {};
            request.body = {};

            response.staticfileServer = this.staticfileServer;

            //clean up resources once the response has been sent out
            response.on('finish', Util.generateCallback(this.onResponseFinish, this, [request, response]));

            if (this.staticfileServer.serve(url, method, headers, response)) return;

            this.parseRequestData(request, url, bufferDetails.buffers);

            if (this.cordinateRoutes(url, method, request, response)) return;

            //send 404 response if router did not resolved
            var httpErrors = this.config.httpErrors;
            this.staticfileServer.serveHttpErrorFile(response, 404, httpErrors.baseDir, httpErrors['404']);
        }

        /**
         * handle request data event
        */

    }, {
        key: 'onRequestData',
        value: function onRequestData(chunk, request, response, bufferDetails) {
            bufferDetails.size += chunk.length;

            if (bufferDetails.size <= this.config.maxBufferSize) bufferDetails.buffers.push(chunk);else request.destroy(new Error('Payload too large'));
        }

        /**
         * handles request events
        */

    }, {
        key: 'onRequest',
        value: function onRequest(request, response) {
            //profile the request time
            request.startTime = Date.now();

            var bufferDetails = { buffers: [], size: 0 };

            //handle on data event
            request.on('data', Util.generateCallback(this.onRequestData, this, [request, response, bufferDetails]));

            //handle on data event
            request.on('end', Util.generateCallback(this.onRequestEnd, this, [request, response, bufferDetails]));
        }

        /**
         * handle server close event
        */

    }, {
        key: 'onClose',
        value: function onClose() {
            Logger.info('connection closed successfully');
        }

        /**
         * handles client error events
         *@param {Error} err - client error
         *@param {net.Socket} socket - the socket object
        */

    }, {
        key: 'onClientError',
        value: function onClientError(err, socket) {
            socket.end('HTTP/1.1 400 Bad Request\r\n\r\n');
        }

        /**
         * handle server error events
         *@param {Error} err - the error event
        */

    }, {
        key: 'onError',
        value: function onError(err) {
            if (err.code === 'EADDRINUSE') Logger.error('Server can\'t be started, address already in use');else Logger.error('Error :- ' + err.code + ' ' + err.message);
        }

        /**
         * handles server listening event
        */

    }, {
        key: 'onListening',
        value: function onListening() {
            var address = this.server.address();
            Logger.info('Server started on port ' + address.port);
        }

        /**
         * binds all event handlers on the server.
        */

    }, {
        key: 'initServer',
        value: function initServer() {
            //handle server listening event
            this.server.on('listening', Util.generateCallback(this.onListening, this))

            //handle on error event
            .on('error', this.onError)

            //handle server close event
            .on('close', this.onClose)

            //handle server client error
            .on('clientError', this.onClientError)

            //handle server request
            .on('request', Util.generateCallback(this.onRequest, this));
        }

        /**
         * starts the server at a given port
         *@param {number} [port=4000] - the port to listen on.
         *@param {Function} [callback] - a callback function to execute once the server starts
         * listening on the given port
        */

    }, {
        key: 'listen',
        value: function listen(port, callback) {
            if (this.listening) {
                Logger.error('Server already started. You must close the server first.');
                return;
            }
            callback = Util.isCallable(callback) ? callback : function () {};
            this.server.listen(port || 4000, callback);
        }

        /**
         * closes the connection
         *@param {Function} [callback] - a callback function to execute once the server closes
        */

    }, {
        key: 'close',
        value: function close(callback) {
            callback = Util.isCallable(callback) ? callback : function () {};
            this.server.close(callback);
        }

        /**
         * returns the bound address that the server is listening on
         *@returns {Object}
        */

    }, {
        key: 'address',
        value: function address() {
            if (this.listening) return this.server.address();else return null;
        }
    }, {
        key: Symbol.toStringTag,
        get: function get$$1() {
            return 'RServer';
        }

        /**
         * returns boolean indicating if the server is listening
        */

    }, {
        key: 'listening',
        get: function get$$1() {
            return this.server.listening;
        }
    }]);
    return _class;
}();

module.exports = _class;
