'use strict';

var RServer = require('./RServer.js');
var RouteWrapper = require('./RouteWrapper.js');

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var RServerApp = function () {

    /**
     *@param {string} [configPath] - user defined configuration file location.
    */
    function RServerApp(configPath) {
        classCallCheck(this, RServerApp);

        this.server = new RServer(configPath);
    }

    /**
     * return instance identity
    */


    createClass(RServerApp, [{
        key: 'options',


        /**
         * performs route rules for http OPTIONS method verb
         *@param {string} url - the route url
         *@param {Function} callback - callback function
         *@param {Object} [options] - optional configuration options
        */
        value: function options(url, callback, _options) {
            this.server.router.options(url, callback, _options);
        }

        /**
         * performs route rules for http HEAD method verb
         *@param {string} url - the route url
         *@param {Function} callback - callback function
         *@param {Object} [options] - optional configuration options
        */

    }, {
        key: 'head',
        value: function head(url, callback, options) {
            this.server.router.head(url, callback, options);
        }

        /**
         * performs route rules for http GET method verb
         *@param {string} url - the route url
         *@param {Function} callback - callback function
         *@param {Object} [options] - optional configuration options
        */

    }, {
        key: 'get',
        value: function get$$1(url, callback, options) {
            this.server.router.get(url, callback, options);
        }

        /**
         * performs route rules for http POST method verb
         *@param {string} url - the route url
         *@param {Function} callback - callback function
         *@param {Object} [options] - optional configuration options
        */

    }, {
        key: 'post',
        value: function post(url, callback, options) {
            this.server.router.post(url, callback, options);
        }

        /**
         * performs route rules for http PUT method verb
         *@param {string} url - the route url
         *@param {Function} callback - callback function
         *@param {Object} [options] - optional configuration options
        */

    }, {
        key: 'put',
        value: function put(url, callback, options) {
            this.server.router.put(url, callback, options);
        }

        /**
         * performs route rules for http DELETE method verb
         *@param {string} url - the route url
         *@param {Function} callback - callback function
         *@param {Object} [options] - optional configuration options
        */

    }, {
        key: 'delete',
        value: function _delete(url, callback, options) {
            this.server.router.delete(url, callback, options);
        }

        /**
         * performs route rules for all http method verbs
         *@param {string} url - the route url
         *@param {Function} callback - callback function
         *@param {Object} [options] - optional configuration options
        */

    }, {
        key: 'all',
        value: function all(url, callback, options) {
            this.server.router.all(url, callback, options);
        }

        /**
         * returns a route wrapper for the given url
         *@returns {RouteWrapper}
        */

    }, {
        key: 'route',
        value: function route(url) {
            return new RouteWrapper(this.server.router, url);
        }

        /**
         * use a middleware
         *@param {Function} middleware - the middleware function
        */

    }, {
        key: 'use',
        value: function use(middleware) {
            this.server.router.use(middleware);
        }

        /**
         * mounts the router (mini-app) on the main app
         *@param {string} baseUrl - the base url
         *@param {Router} router - the router (mini-app)
        */

    }, {
        key: 'mount',
        value: function mount(baseUrl, router) {
            this.server.mount(baseUrl, router);
        }

        /**
         * starts the serve to listen on a given port
         *@param {number} [port=4000] - the port to listen on. defaults to port 4000
         *@param {Function} [callback] - a callback function to execute once the server starts
         * listening
        */

    }, {
        key: 'listen',
        value: function listen(port, callback) {
            this.server.listen(port, callback);
        }

        /**
         * closes the connection
         *@param {Function} [callback] - a callback function to execute once the server gets closed
        */

    }, {
        key: 'close',
        value: function close(callback) {
            this.server.close(callback);
        }

        /**
         * returns boolean value indicating if the app is listening for request
        */

    }, {
        key: 'address',


        /**
         * returns the bound address that the app is listening on
         *@returns {Object}
        */
        value: function address() {
            return this.server.address();
        }
    }, {
        key: Symbol.toStringTag,
        get: function get$$1() {
            return 'RserverApp';
        }
    }, {
        key: 'listening',
        get: function get$$1() {
            return this.server.listening;
        }
    }]);
    return RServerApp;
}();

module.exports = RServerApp;
