'use strict';

var Util = require('./Util.js');
var RouteWrapper = require('./RouteWrapper.js');

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var Router = function () {

    /**
     * creates router
     *@param {boolean} [inheritMiddlewares=true] - boolean value indicating if parent
     * middlewares should be inherited. defaults to true
    */
    function Router(inheritMiddlewares) {
        classCallCheck(this, Router);

        this.routes = {
            options: [],
            head: [],
            get: [],
            post: [],
            put: [],
            delete: [],
            all: []
        };
        this.middlewares = [];
        this.inheritMiddlewares = inheritMiddlewares === false ? false : true;
    }

    /**
     * return object identity
    */


    createClass(Router, [{
        key: 'options',


        /**
         * performs route rules for http OPTIONS method verb
         *@param {string} url - the route url
         *@param {Function} callback - callback function
         *@param {Object} [options] - optional configuration options
        */
        value: function options(url, callback, _options) {
            this.routes.options.push([url, callback, _options]);
        }

        /**
         * performs route rules for http HEAD method verb
         *@param {string} url - the route url
         *@param {Function} callback - callback function
         *@param {Object} [options] - optional configuration options
        */

    }, {
        key: 'head',
        value: function head(url, callback, options) {
            this.routes.head.push([url, callback, options]);
        }

        /**
         * performs route rules for http GET method verb
         *@param {string} url - the route url
         *@param {Function} callback - callback function
         *@param {Object} [options] - optional configuration options
        */

    }, {
        key: 'get',
        value: function get$$1(url, callback, options) {
            this.routes.get.push([url, callback, options]);
        }

        /**
         * performs route rules for http POST method verb
         *@param {string} url - the route url
         *@param {Function} callback - callback function
         *@param {Object} [options] - optional configuration options
        */

    }, {
        key: 'post',
        value: function post(url, callback, options) {
            this.routes.post.push([url, callback, options]);
        }

        /**
         * performs route rules for http PUT method verb
         *@param {string} url - the route url
         *@param {Function} callback - callback function
         *@param {Object} [options] - optional configuration options
        */

    }, {
        key: 'put',
        value: function put(url, callback, options) {
            this.routes.put.push([url, callback, options]);
        }

        /**
         * performs route rules for http DELETE method verb
         *@param {string} url - the route url
         *@param {Function} callback - callback function
         *@param {Object} [options] - optional configuration options
        */

    }, {
        key: 'delete',
        value: function _delete(url, callback, options) {
            this.routes.delete.push([url, callback, options]);
        }

        /**
         * performs route rules for all http method verbs
         *@param {string} url - the route url
         *@param {Function} callback - callback function
         *@param {Object} [options] - optional configuration options
        */

    }, {
        key: 'all',
        value: function all(url, callback, options) {
            this.routes.all.push([url, callback, options]);
        }

        /**
         * returns a route wrapper for the given url
         *@returns {RouteWrapper}
        */

    }, {
        key: 'route',
        value: function route(url) {
            return new RouteWrapper(this, url);
        }

        /**
         * use a middleware
         *@param {Function} middleware - the middleware function
        */

    }, {
        key: 'use',
        value: function use(middleware) {
            if (Util.isCallable(middleware)) this.middlewares.push(middleware);
        }
    }, {
        key: Symbol.toStringTag,
        get: function get$$1() {
            return 'Router';
        }
    }]);
    return Router;
}();

module.exports = Router;
