'use strict';

var Util = require('./Util.js');

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var slicedToArray = function () {
  function sliceIterator(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"]) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  return function (arr, i) {
    if (Array.isArray(arr)) {
      return arr;
    } else if (Symbol.iterator in Object(arr)) {
      return sliceIterator(arr, i);
    } else {
      throw new TypeError("Invalid attempt to destructure non-iterable instance");
    }
  };
}();

/**
 * Routing engine module.
 * Handles Routing. supports parameter catching and allows data type enforcement
*/

var RoutingEngine = function () {

    /**
     *@param {string} url - the request url
     *@param {string} method - the request method
     *@param {http.IncomingMessage} request - the request instance
     *@param {RServerResponse} response - the response instance
     *@param {Array} [middlewares] - Array of middlewares
    */
    function RoutingEngine(url, method, request, response, middlewares) {
        classCallCheck(this, RoutingEngine);


        this.resolved = false;
        this.request = request;
        this.response = response;
        this.middlewares = Util.isArray(middlewares) ? middlewares : [];

        this.url = url.toLowerCase().replace(/[#?].*$/, '').replace(/^\/+/, '').replace(/\/+$/, '');
        this.method = method.toUpperCase();

        this.params = []; //create a simple [paramName, paramValue] tuple
    }

    /**
     * return class identifier
    */


    createClass(RoutingEngine, [{
        key: 'use',


        /**
         * sets or overrides the existing middlewares
         *@param {Array} middlewares - array of middlewares
        */
        value: function use(middlewares) {
            if (Util.isArray(middlewares)) this.middlewares = middlewares;
        }

        /**
         * runs the routes template callback function
         *@param {Function} - route callback function
        */

    }, {
        key: 'run',
        value: function run(callback) {
            var cont = false,
                next = function next() {
                cont = true;
            };

            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError = undefined;

            try {
                for (var _iterator = this.middlewares[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                    var middleware = _step.value;

                    cont = false;
                    Util.runSafe(middleware, null, [this.request, this.response, next]);

                    if (cont && !this.response.finished) continue;
                    //if middleware failed to end the response, end it
                    else if (!this.response.finished) this.response.end();

                    return;
                }
            } catch (err) {
                _didIteratorError = true;
                _iteratorError = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion && _iterator.return) {
                        _iterator.return();
                    }
                } finally {
                    if (_didIteratorError) {
                        throw _iteratorError;
                    }
                }
            }

            var values = [];
            var _iteratorNormalCompletion2 = true;
            var _didIteratorError2 = false;
            var _iteratorError2 = undefined;

            try {
                for (var _iterator2 = this.params[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                    var _ref = _step2.value;

                    var _ref2 = slicedToArray(_ref, 2);

                    var value = _ref2[1];

                    values.push(value);
                }
            } catch (err) {
                _didIteratorError2 = true;
                _iteratorError2 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion2 && _iterator2.return) {
                        _iterator2.return();
                    }
                } finally {
                    if (_didIteratorError2) {
                        throw _iteratorError2;
                    }
                }
            }

            Util.runSafe(callback, null, [this.request, this.response].concat(values));
        }

        /**
         * decomposes the template token into data type and name
         *@param {string} routeToken - the route template token to be decomposed
         *@param {string} pathToken - the corresponding path token for this route template token
         *@returns {Object} returns object containing dataType and name
         * keys
        */

    }, {
        key: 'deComposeRouteToken',
        value: function deComposeRouteToken(routeToken, pathToken) {
            var storeAsParam = false;

            if (/\{([-\w:]+)\}/.exec(routeToken)) {
                storeAsParam = true;
                routeToken = RegExp.$1;
            }

            var _ref3 = routeToken.indexOf(':') > -1 ? routeToken.split(':') : ['', routeToken],
                _ref4 = slicedToArray(_ref3, 2),
                dataType = _ref4[0],
                name = _ref4[1],
                value = pathToken;

            switch (dataType.toLowerCase()) {
                case 'int':
                case 'integer':
                    value = Number.parseInt(pathToken);
                    break;
                case 'float':
                case 'double':
                case 'number':
                    value = Number.parseFloat(pathToken);
                    break;
                case 'bool':
                case 'boolean':
                    value = ['0', 'false', ''].includes(pathToken.toString().toLowerCase()) ? false : true;
                    break;
            }

            if (Number.isNaN(value)) value = 0;

            if (storeAsParam) this.params.push([name, value]);

            return { name: name, value: value, dataType: dataType };
        }

        /**
         * analyses base url and creates a regex for it. then matches and returns the result
         *@param {string} routeUrl - a route's url
         *@returns {boolean}
        */

    }, {
        key: 'matchUrl',
        value: function matchUrl(routeUrl) {
            /**create matching regular expression*/
            var tokens = routeUrl ? routeUrl.split('/') : [],
                pattern = tokens.map(function (token) {
                var pattern = '';
                if (/^\{[\w:-]+\}$/.exec(token)) pattern = '[^/]+';else if (/^\{[\w:-]+\}\?$/.exec(token)) pattern = '([^/]+)?';else if (token === '*') pattern = '.*';else pattern = token.replace(/\\-/g, '-').replace(/-/g, '\\-');

                return pattern;
            }).join('/');

            var regex = new RegExp('^' + pattern + '$', 'i'); //regex is case insensitive
            return regex.test(this.url);
        }

        /**
         *@param {Object} [options] - optional configuration options
         *@param {Array} [options.methods] - array of methods allowed
         *@returns {boolean}
        */

    }, {
        key: 'validateOptions',
        value: function validateOptions(options) {
            options = Util.isPlainObject(options) ? options : {};
            var result = true;

            //validate request method. the request method should be among the options.methods item.
            if (Util.isArray(options.methods)) {
                var method = this.method;
                result = options.methods.some(function (testMethod) {
                    return typeof testMethod === 'string' && testMethod.toUpperCase() === method;
                });
            }
            return result;
        }

        /**
         *@description - checks if the request method is ok and that callback is a function
         *@param {Function} callback - the callback function
         *@param {string} [overrideMethod] - method to use
         *@returns {boolean}
        */

    }, {
        key: 'validateRoute',
        value: function validateRoute(callback, overrideMethod) {
            return Util.isCallable(callback) && (!overrideMethod || this.method === overrideMethod.toUpperCase());
        }

        /**
         * processes the route
         *@param {string} routeUrl - the route's url
         *@param {Function} callback - callback function
         *@param {Object} [options] - optional configuration options
         *@param {Array} [options.methods] - array of methods allowed
         *@param {string} [overrideMethod] - a string indicating the only method allowed for the route
         *@returns {boolean}
        */

    }, {
        key: 'process',
        value: function process(routeUrl, callback, options, overrideMethod) {
            if (this.resolved) return;

            this.params = []; //reset the params tuple
            routeUrl = routeUrl.toLowerCase().replace(/^\/+/, '').replace(/\/+$/, '');

            if (!this.validateRoute(callback, overrideMethod) || !this.validateOptions(options) || !this.matchUrl(routeUrl)) return;

            var pathTokens = this.url !== '' ? this.url.split('/') : [],
                routeTokens = routeUrl !== '' ? routeUrl.split('/') : [];

            var i = -1;
            while (++i < routeTokens.length) {
                var pathToken = pathTokens[i],
                    routeToken = routeTokens[i];

                if (routeToken === '*') {
                    //final route token. add remaining url and pass it to the callback
                    this.params.push(['asterisk', pathTokens.slice(i).join('/')]);
                    break;
                }
                this.deComposeRouteToken(routeToken, pathToken);
            }

            this.resolved = true;
            this.run(callback);
        }

        /**
         * performs route rules for all http method verbs
         *@param {string} routeUrl - the route url
         *@param {Function} - callback function
         *@param {Object} [options] - optional configuration options
         *@param {Array} [options.methods] - array of methods allowed
        */

    }, {
        key: 'all',
        value: function all(routeUrl, callback, options) {
            this.process(routeUrl, callback, options, '');
        }

        /**
         * performs route rules for only GET request method verb
         *@param {string} routeUrl - the route url
         *@param {Function} - callback function
         *@param {Object} [options] - optional configuration options
        */

    }, {
        key: 'get',
        value: function get$$1(routeUrl, callback, options) {
            this.process(routeUrl, callback, options, 'GET');
        }

        /**
         * performs route rules for only HEAD request method verb
         *@param {string} routeUrl - the route url
         *@param {Function} - callback function
         *@param {Object} [options] - optional configuration options
        */

    }, {
        key: 'head',
        value: function head(routeUrl, callback, options) {
            this.process(routeUrl, callback, options, 'HEAD');
        }

        /**
         * performs route rules for only OPTIONS request method verb
         *@param {string} routeUrl - the route url
         *@param {Function} - callback function
         *@param {Object} [options] - optional configuration options
        */

    }, {
        key: 'options',
        value: function options(routeUrl, callback, _options) {
            this.process(routeUrl, callback, _options, 'OPTIONS');
        }

        /**
         * performs route rules for only DELETE method verb
         *@param {string} routeUrl - the route url
         *@param {Function} - callback function
         *@param {Object} [options] - optional configuration options
        */

    }, {
        key: 'delete',
        value: function _delete(routeUrl, callback, options) {
            this.process(routeUrl, callback, options, 'DELETE');
        }

        /**
         * performs route rules for only POST request method verb
         *@param {string} routeUrl - the route url
         *@param {Function} - callback function
         *@param {Object} [options] - optional configuration options
        */

    }, {
        key: 'post',
        value: function post(routeUrl, callback, options) {
            this.process(routeUrl, callback, options, 'POST');
        }

        /**
         * performs route rules for only PUT request method verb
         *@param {string} routeUrl - the route url
         *@param {Function} - callback function
         *@param {Object} [options] - optional configuration options
        */

    }, {
        key: 'put',
        value: function put(routeUrl, callback, options) {
            this.process(routeUrl, callback, options, 'PUT');
        }
    }, {
        key: Symbol.toStringTag,
        get: function get$$1() {
            return 'RoutingEngine';
        }
    }]);
    return RoutingEngine;
}();

module.exports = RoutingEngine;
